﻿namespace CoilTurnsCalculator
{



    #region Using Statements:



    using System;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms;
    using System.Threading.Tasks;
    using System.Collections.Generic;



    #endregion



    /// <summary>
    /// Coil Calculations for www.aboveunity.com Members.
    /// </summary>
    public partial class AboveunityMemberCalculator : Form
    {



        #region Fields:



        private double SenseResistorVoltage = 0;



        private double SenseResistorResistance = 0;



        private double InputVoltage = 0;



        private double Answer = 0;



        #endregion



        #region Properties:



        /// <summary>
        /// The turns ratio of a transformer is sometimes represented by the letter a: Np / Ns
        /// </summary>
        double a { get; set; }



        /// <summary>
        /// The Turns N on the Primary Coil.
        /// </summary>
        double Np { get; set; }



        /// <summary>
        /// The Turns N on the Secondary Coil.
        /// </summary>
        double Ns { get; set; }



        /// <summary>
        /// The Inductance L on the Primary Coil.
        /// </summary>
        double Lp { get; set; }



        /// <summary>
        /// The Inductance L on the Secondary Coil.
        /// </summary>
        double Ls { get; set; }



        /// <summary>
        /// The Inductance Ratio Lr of the Transformer.
        /// </summary>
        double Lr { get; set; }



        /// <summary>
        /// The Voltage V  on the Primary Coil.
        /// </summary>
        double Vp { get; set; }



        /// <summary>
        /// The Voltage V on the Secondary Coil.
        /// </summary>
        double Vs { get; set; }



        /// <summary>
        /// The Impedance Z on the Primary Coil.
        /// </summary>
        double Zp { get; set; }



        /// <summary>
        /// The Impedance Z on the Secondary Coil.
        /// </summary>
        double Zs { get; set; }



        /// <summary>
        /// The Impedance Ratio Zr of the Transformer.
        /// </summary>
        double Zr { get; set; }



        /// <summary>
        /// The Magnetic Field B in Tesla's, 10000 Gauss is 1 Tesla.
        /// </summary>
        double B { get; set; }



        /// <summary>
        /// The Current in the Solenoid, in Amperes.
        /// </summary>
        double I { get; set; }



        /// <summary>
        /// k is the Relative Permeability of the Core.
        /// </summary>
        double k { get; set; }



        /// <summary>
        /// The Length of the Solenoid in Meters.
        /// </summary>
        double L { get; set; }



        /// <summary>
        /// The Number of Turns carrying the Current.
        /// </summary>
        double N { get; set; }



        /// <summary>
        /// The Turn Density per Meter. n = N/L
        /// </summary>
        double TurnDensity { get; set; }



        /// <summary>
        /// The Relative Permeability, this is the ratio of the permeability of a specific medium to the permeability of free space µ0:
        /// </summary>
        double RelativePermeability { get; set; }



        /// <summary>
        /// The Permeability of Free Space, or Vacuum Permeability.
        /// <para>
        /// μ0 = 4π×10−7 H/m
        /// </para>
        /// </summary>
        double Permeability { get; set; }



        #endregion




        public AboveunityMemberCalculator()
        {

            InitializeComponent();

            // 
            this.srVComboBox.SelectedIndexChanged += new System.EventHandler(this.SrVComboBox_SelectedIndexChanged);
            this.inVComboBox.SelectedIndexChanged += new System.EventHandler(this.InVComboBox_SelectedIndexChanged);
            this.ansComboBox.SelectedIndexChanged += new System.EventHandler(this.AnsComboBox_SelectedIndexChanged);
        }



        private void Form1_Load(object sender, EventArgs e)
        {

            // Set Wave Type:
            WaveTypeComboBox.SelectedIndex = 0;


            #region Sensing Resistor Region:



            srVComboBox.Items.Add("Volts");
            srVComboBox.Items.Add("milliVolts");
            srVComboBox.SelectedItem = "Volts";

            srRComboBox.Items.Add("Ohms");
            srRComboBox.SelectedItem = "Ohms";

            inVComboBox.Items.Add("Volts");
            inVComboBox.Items.Add("milliVolts");
            inVComboBox.SelectedItem = "Volts";

            ansComboBox.Items.Add("Watts");
            ansComboBox.Items.Add("milliWatts");
            ansComboBox.SelectedItem = "Watts";

            label46.Text = "The Current through a resistor can be calculated by I = V / R. \n \n " +
                "With a sense Resistor we can measure input current on a scope. \n Typically " +
                "these resistors are very small in vaule so as to not affect \n the circuits Operation. \n " +
                "\n From 1 ohm down to 0.0001 Ohms can be used and either way \n each side of these " +
                "figures. \n \n With the Current now measured, flowing through the Circuit we can \n " +
                "calculate the power used by P = V * I. \n \n Power P (typically measured in Watts) = " +
                "V (Volts) * I (Current) \n ";



            #endregion
        }



        /// <summary>
        /// Calculate Coil Turns Ratios using Open Inductance Measurement.
        /// </summary>
        private void CalculateTurnsRatiosButton_Click(object sender, EventArgs e)
        {

            // Get Inputs:
            if (!double.TryParse(PrimaryInductanceTextBox.Text, out double PrimaryInductance))
            {
                MessageBox.Show("Error: Please check the Primary Inductance value!", "Error: Primary Inductance:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                return;
            }

            Lp = PrimaryInductance;

            if (!double.TryParse(SecondaryInductanceTextBox.Text, out double SecondaryInductance))
            {
                MessageBox.Show("Error: Please check the Secondary Inductance value!", "Error: Secondary Inductance:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                return;
            }

            Ls = SecondaryInductance;

            if (!double.TryParse(InputVoltageTextBox.Text, out double Voltage))
            {
                MessageBox.Show("Error: Please check the Input Voltage value!", "Error: Input Voltage:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                return;
            }

            Vp = Voltage;

            if (!double.TryParse(LoadImpedanceTextBox.Text, out double LoadImpedance))
            {
                MessageBox.Show("Error: Please check the Output Impedance value!", "Error: Output Impedance:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                return;
            }

            if ((Lp / Ls) > 1.0D)
            {

                // Calculate the Turns Ratio: a = sqrt( Lp / Ls )
                a = Math.Sqrt(Lr = (Lp / Ls));
                Math.Round(a, 4);

                // Calculate the Impedance Ratio: Zr = a^2
                Zr = Math.Pow(a, 2.0);
                Zr = Math.Round(Zr, 4);

                // 
                Vs = (a / Vp);
                Vs = Math.Round(Vs, 4);

                Np = a;
                Ns = 1;

                Lp = Zr;
                Ls = 1;

                Zp = Zr;
                Zs = 1;
            }
            else
            {

                // Calculate the Turns Ratio: a = sqrt( Lp / Ls )
                a = Math.Sqrt(Lr = (Ls / Lp));
                a = Math.Round(a, 4);

                // Calculate the Impedance Ratio: Zr = a^2
                Zr = Math.Pow(a, 2.0);
                Zr = Math.Round(Zr, 4);

                // 
                Vs = (a * Vp);
                Vs = Math.Round(Vs, 4);

                Np = 1;
                Ns = a;

                Lp = 1;
                Ls = Zr;

                Zp = 1;
                Zs = Zr;
            }

            // Display Ratio's:
            NrLabel.Text = Math.Round(Np, 3) + " : " + Math.Round(Ns, 3);
            LrLabel.Text = Math.Round(Lp, 3) + " : " + Math.Round(Ls, 3);
            ZrLabel.Text = Math.Round(Zp, 3) + " : " + Math.Round(Zs, 3);
            CrLabel.Text = Math.Round(Np, 3) + " : " + Math.Round(Ns, 3);
            VrLabel.Text = Math.Round(Ns, 3) + " : " + Math.Round(Np, 3);


            OvLabel.Text = Vs + " Volts";
            OimpLabel.Text = Math.Round((Zr * LoadImpedance), 4) + " Ohms";
            OcLabel.Text = Math.Round(Vs / (Zr * LoadImpedance), 4) + " Amps";
        }



        /// <summary>
        /// I still do tests to understand your experiment, but everything has to do
        /// with magnetic permeability and standing waves.
        /// The turns ratios must be quadratic because the inductance is.
        /// This means that the turns ratios will be the root of 0.75, 0.5 or 0.25 of lambda for
        /// one of your inductors N2 = SQR(n) N1, where n is one of the previous numbers.
        /// See: http://www.aboveunity.com/thread/romanian-zpm-zero-point-module/?order=all#comment-8c676694-b094-4254-adb3-aaab014b8606
        /// </summary>
        private void CalculateQuadraticTurnsButton_Click(object sender, EventArgs e)
        {


            if (!double.TryParse(QuadraticCoilTurnsTextBox.Text, out double QuadraticCoilTurns))
            {

                MessageBox.Show("Error: Please check the Input Voltage value!", "Error: Input Voltage:", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1, MessageBoxOptions.RightAlign);

                return;
            }

            // Turns:
            double L1Turns = QuadraticCoilTurns;
            double L2Turns = 0.0D;

            // Wave Types:
            double QatarWave = 0.25;

            // Chosen Wave: λ
            double Lambda = (QatarWave * (WaveTypeComboBox.SelectedIndex + 1));

            // Do Math:
            L2Turns = Math.Sqrt(Lambda) * L1Turns;

            // Display Result:
            QuadraticTurnsTextBox.Text = L2Turns.ToString();
        }



        /// <summary>
        /// Calculate Magnetic Field B.
        /// </summary>
        private void CalculateMagneticFieldButton_Click(object sender, EventArgs e)
        {

            if (!double.TryParse(LTextBox.Text, out double ll))
            {
                MessageBox.Show("Length Error.");
                return;
            }

            if (!double.TryParse(NTextBox.Text, out double ln))
            {
                MessageBox.Show("Turns Error.");
                return;
            }

            if (!double.TryParse(KTextBox.Text, out double lk))
            {
                MessageBox.Show("Permeability Error");
                return;
            }

            if (!double.TryParse(ITextBox.Text, out double li))
            {
                MessageBox.Show("Current Error");
                return;
            }


            L = ll;
            N = ln;
            k = lk;
            I = li;

            // Permeability of Free Space:
            Permeability = (4 * Math.PI) * (Math.Pow(10, -7));

            // The relative permeability of magnetic iron is around 200.
            // k = 200;

            // Relative Permeability:
            RelativePermeability = k * Permeability;

            // Turns:
            // N = 100;

            // Length in Meters:
            // L = 0.01;

            // Turn Density:
            TurnDensity = N / L;

            // Curent:
            // I = 1.0;

            // Magnetic Field:
            B = RelativePermeability * TurnDensity * I;

            // B = 2.513274122871835 Tesla = 25132.74122871835 gauss.
            BTextBox.Text = Math.Round(B, 7).ToString();
            GTextBox.Text = Math.Round((B * 10000), 7).ToString();
        }



        /// <summary>
        /// Calculate Phase Angle.
        /// </summary>
        private void CalculatePhaseAngleButton_Click(object sender, EventArgs e)
        {


            double FullCycle = 360;
            double DegPerTimePeriod = 0;
            double WaveLength = 0;
            double WaveOffSet = 0;

            try
            {

                WaveLength = double.Parse(this.WaveLengthTextBox.Text);
                WaveOffSet = double.Parse(this.WaveOffSetTextBox.Text);
            }
            catch
            {

                return;
            }

            DegPerTimePeriod = WaveOffSet / WaveLength;

            this.AnswerTextBox.Text = Math.Round((DegPerTimePeriod * FullCycle), 1).ToString();
        }



        /// <summary>
        /// Calculate AC Power.
        /// </summary>
        private void CalculateACPowerButton_Click(object sender, EventArgs e)
        {


            double v = 0;
            double i = 0;
            double ph = 0;

            try
            {

                v = Convert.ToDouble(this.VoltageTextBox.Text);
                i = Convert.ToDouble(this.CurrentTextBox.Text);
                ph = Convert.ToDouble(this.PhaseAngleTextBox.Text);
            }
            catch
            {

                String message = "Value entered is incorrect!";
                String caption = "Please check Entrys";
                MessageBoxButtons buttons = MessageBoxButtons.OK;
                MessageBoxIcon icon = MessageBoxIcon.Asterisk;

                MessageBox.Show(message, caption, buttons, icon);

                return;
            }

            //////////////////////////////////////////////////////////////
            // See: https://www.youtube.com/watch?v=RU6jtai4wys         //
            //////////////////////////////////////////////////////////////

            double radians = DegreeToRadian(ph);

            double CosAngle = Math.Cos(radians);
            double SinAngle = Math.Sin(radians);

            CosAngle = Math.Round(CosAngle, 7);
            SinAngle = Math.Round(SinAngle, 7);

            this.WattsTextBox.Text = ((v * i) * CosAngle).ToString();
            this.VATextBox.Text = (v * i).ToString();
            this.VArTextBox.Text = ((v * i) * SinAngle).ToString();
            this.PFTextBox.Text = (CosAngle).ToString();
        }



        /// <summary>
        /// Clear.
        /// </summary>
        private void ClearSenseResistorCalculatorButton_Click(object sender, EventArgs e)
        {

            srVTextBox.Text = "";
            srRTextBox.Text = "";
            inVTextBox.Text = "";
            SenseResistorAnswerTextBox.Text = "";
        }



        /// <summary>
        /// Calculate Power using a Sense Resistor.
        /// </summary>
        private void CalculateSenseResistorCalculatorButton_Click(object sender, EventArgs e)
        {

            SenseResistorAnswerTextBox.Text = Calculate(ansComboBox.SelectedItem.ToString()).ToString();
        }



        /// <summary>
        /// Calculate Power using a Sense Resistor.
        /// </summary>
        private double Calculate(string watts)
        {

            try
            {

                SenseResistorVoltage = Convert.ToDouble(srVTextBox.Text);
                SenseResistorResistance = Convert.ToDouble(srRTextBox.Text);
                InputVoltage = Convert.ToDouble(inVTextBox.Text);

                if (srVComboBox.SelectedItem.ToString() == "milliVolts")
                {
                    SenseResistorVoltage = SenseResistorVoltage / 1000;
                }

                if (inVComboBox.SelectedItem.ToString() == "milliVolts")
                {
                    InputVoltage = InputVoltage / 1000;
                }

                double wattsDissipatedBySenseResistor = ((SenseResistorVoltage / SenseResistorResistance) * SenseResistorVoltage);

                double wattsDissipatedByLoad = ((SenseResistorVoltage / SenseResistorResistance) * InputVoltage) - wattsDissipatedBySenseResistor;

                wattsDissipatedByLoad = Math.Round(wattsDissipatedByLoad, 4);

                if (watts == "milliWatts") wattsDissipatedByLoad = wattsDissipatedByLoad * 1000;

                notify.Text = watts + " dissipated in Load: " + wattsDissipatedByLoad + " Watts + " + Math.Round(wattsDissipatedBySenseResistor, 4) + " Watts dissipated in Sense Resistor.";

                Answer = (wattsDissipatedByLoad + wattsDissipatedBySenseResistor);

                return Answer;
            }
            catch
            {

                notify.Text = "Please check your input data...";
                return 0.0D;
            }
        }



        /// <summary>
        /// ComboBox Index Changed Event Handler.
        /// </summary>
        private void InVComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            //1 volt = 1000 millivolts
            //1 volt = 1 000 000 microvolts

            if (inVComboBox.SelectedItem == "Volts" && inVTextBox.Text != "")
            {
                double tmp = 0;
                tmp = Convert.ToDouble(inVTextBox.Text);
                InputVoltage = (tmp / 1000.0D);
                inVTextBox.Text = InputVoltage.ToString();
            }

            if (inVComboBox.SelectedItem == "milliVolts" && inVTextBox.Text != "")
            {
                double tmp = 0;
                tmp = Convert.ToDouble(inVTextBox.Text);
                InputVoltage = (tmp * 1000.0D);
                inVTextBox.Text = InputVoltage.ToString();
            }
        }



        /// <summary>
        /// ComboBox Index Changed Event Handler.
        /// </summary>
        private void SrVComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            //1 volt = 1000 millivolts
            //1 volt = 1 000 000 microvolts

            if (srVComboBox.SelectedItem == "Volts" && srVTextBox.Text != "")
            {
                double tmp = 0;
                tmp = Convert.ToDouble(srVTextBox.Text);
                SenseResistorVoltage = (tmp / 1000.0D);
                srVTextBox.Text = SenseResistorVoltage.ToString();
            }

            if (srVComboBox.SelectedItem == "milliVolts" && srVTextBox.Text != "")
            {
                double tmp = 0;
                tmp = Convert.ToDouble(srVTextBox.Text);
                SenseResistorVoltage = (tmp * 1000.0D);
                srVTextBox.Text = SenseResistorVoltage.ToString();
            }
        }



        /// <summary>
        /// ComboBox Index Changed Event Handler.
        /// </summary>
        private void AnsComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {

            //1 volt = 1000 millivolts
            //1 volt = 1 000 000 microvolts
            SenseResistorAnswerTextBox.Text = Calculate(ansComboBox.SelectedItem.ToString()).ToString();
        }



        /// <summary>
        /// Convert Degrees to Radians.
        /// </summary>
        /// <param name="angle">The Angle in Degrees.</param>
        /// <returns>The Angle in Radians.</returns>
        private double DegreeToRadian(double angle)
        {
            return Math.PI * angle / 180.0;
        }



        /// <summary>
        /// Convert Radians to Degrees.
        /// </summary>
        /// <param name="angle">The Angle in Radians.</param>
        /// <returns>The Angle in Degrees.</returns>
        private double RadianToDegree(double angle)
        {
            return angle * (180.0 / Math.PI);
        }



        /// <summary>
        /// Calculate the Wavelength.
        /// </summary>
        private void CalculateWavelengthButton_Click(object sender, EventArgs e)
        {

            // Trt get entries:
            double.TryParse(VelocityTextBox.Text, out double Velocity);
            double.TryParse(FrequencyTextBox.Text, out double Frequency);
            double.TryParse(Wavelength1TextBox.Text, out double Wavelength);

            if (Velocity == 0.0D & Frequency == 0.0D)
            {

                MessageBox.Show("Error: Not enough values have been entered!",
                                "We need two Values:", 
                                MessageBoxButtons.OK, 
                                MessageBoxIcon.Error, 
                                MessageBoxDefaultButton.Button1, 
                                MessageBoxOptions.RightAlign);
                return;
            }
            else if (Wavelength == 0.0D & Frequency == 0.0D)
            {

                MessageBox.Show("Error: Not enough values have been entered!",
                                "We need two Values:",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error,
                                MessageBoxDefaultButton.Button1,
                                MessageBoxOptions.RightAlign);
                return;
            }
            else if (Velocity == 0.0D & Wavelength == 0.0D)
            {

                MessageBox.Show("Error: Not enough values have been entered!",
                                "We need two Values:",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error,
                                MessageBoxDefaultButton.Button1,
                                MessageBoxOptions.RightAlign);
                return;
            }


            if (Velocity > 0.0D & Frequency > 0.0D)
                Wavelength1TextBox.Text = Math.Round((Velocity / Frequency), 2).ToString();
            else if (Wavelength > 0.0D & Frequency > 0.0D)
                VelocityTextBox.Text = Math.Round((Frequency * Wavelength), 2).ToString();
            else if (Velocity > 0.0D & Wavelength > 0.0D)
                FrequencyTextBox.Text = Math.Round((Velocity / Wavelength), 2).ToString();
        }



        /// <summary>
        /// Calculate the LCR Resonance.
        /// </summary>
        private void CalculateLCRButton_Click(object sender, EventArgs e)
        {

            // Local vars:
            double Frequency = 0.0D;
            double Reactance = 0.0D;
            double QFactor = 0.0D;
            double Bandwidth = 0.0D;

            // Trt get entries:
            double.TryParse(InductanceLCRTextBox.Text, out double Inductance);
            double.TryParse(CapacitanceLCRTextBox.Text, out double Capacitance);
            double.TryParse(ResistanceLCRTextBox.Text, out double Resistance);


            if (Capacitance == 0.0D & Resistance == 0.0D & Resistance == 0.0D)
            {

                MessageBox.Show("Error: Not enough values have been entered!",
                                "We need three Values:",
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error,
                                MessageBoxDefaultButton.Button1,
                                MessageBoxOptions.RightAlign);
                return;
            }


            if (Capacitance > 0.0D && Inductance > 0.0D && Resistance > 0.00000000001)
            {

                Frequency = 1 / (2 * Math.PI * Math.Sqrt(Inductance * Capacitance));
                Reactance = (2 * Math.PI * Frequency * Inductance);
                QFactor = Resistance / Reactance;
                Bandwidth = Frequency / QFactor;

                ResonanceLCRTextBox.Text = Frequency.ToString();
                ReactanceLCRTextBox.Text = Reactance.ToString();
                QualityFactorLCRTextBox.Text = QFactor.ToString();
                BandwidthLCRTextBox.Text = Bandwidth.ToString();
            }
        }



        /// <summary>
        /// Calculate the AC Circuit Impedance.
        /// </summary>
        private void CalculateButton_Click(object sender, EventArgs e)
        {


            this.richTextBox4.Clear();

            double Volts = 0;
            double Amps = 0;
            double Frequency = 0;

            try
            {

                Volts = double.Parse(this.ACVoltageTextBox.Text);
                Amps = double.Parse(this.ACCurrentTextBox.Text);
                Frequency = double.Parse(this.ACFrequencyTextBox.Text);
            }
            catch
            {
                MessageBox.Show("Input Value Error - Check your Values...");

                return;
            }

            double Resistance = (Volts / Amps);
            double Ohms = Math.Sqrt((Math.Pow(Volts, 2) + Math.Pow(Amps, 2)));
            double PhaseAngle = RadianToDegree(Math.Atan2(Volts, Amps));
            double Real = (Resistance * Math.Cos(DegreeToRadian(PhaseAngle)));
            double XL = (Resistance * Math.Sin(DegreeToRadian(PhaseAngle)));
            double Impedance = Math.Sqrt((Math.Pow(Real, 2) + Math.Pow(XL, 2)));
            double Inductance = XL / (2 * Math.PI * Frequency);
            double Xc = 2 * Math.PI * Frequency * Inductance;
            double Capacitance = 1 / (2 * Math.PI * Frequency * Xc);


            this.richTextBox4.AppendText("Impedance (Z): " + Resistance + " in Ohms. (Ω)" + Environment.NewLine);

            // +j is inductive ...(current lagging)
            this.richTextBox4.AppendText("Impedance (Z): " + Math.Round(Real, 3) + "+j" + Math.Round(XL, 3) + " in Ohms. (Ω)" + Environment.NewLine);

            // -j is capacitive current leading
            // this.richTextBox1.AppendText("Impedance (Z): " + Math.Round(Real, 3) + "-j" + Math.Round(Xc, 3) + " in Ohms. (Ω)" + Environment.NewLine);

            this.richTextBox4.AppendText("Inductive Reactance (XL): " + Math.Round(XL, 3) + " in Ohms. (Ω)" + Environment.NewLine);
            this.richTextBox4.AppendText("Capacitive Reactance (XC): " + Xc + " in Ohms. (Ω)" + Environment.NewLine);
            this.richTextBox4.AppendText("Real Resistance: " + Real + " in Ohms. (Ω)" + Environment.NewLine);
            this.richTextBox4.AppendText("Phase Angle (θ): " + PhaseAngle + " degrees. (Φ)" + Environment.NewLine);
            this.richTextBox4.AppendText("Inductance (L): " + Inductance + " Henries. (H)" + Environment.NewLine);
            this.richTextBox4.AppendText("LC Network Capacitance: " + (Capacitance * 10e-6) + " in Farads. (F)" + Environment.NewLine);
        }
    }
}
